{% load i18n %}

{% block page_header %}
  <div class='page-header'>
      <h1>
          {% trans "Application details" %}: {{ name }}
      </h1>

      <h3>
          <a href=" {% url 'horizon:murano:catalog:index' %}">
              {% trans "Applications" %}
          </a> > {{ name }}
      </h3>
  </div>
{% endblock %}
